package benefactorkindboxhandler

import (
	"net/http"

	param "git.gocasts.ir/ebhomengo/niki/param/benefactor/kind_box"
	httpmsg "git.gocasts.ir/ebhomengo/niki/pkg/http_msg"
	echo "github.com/labstack/echo/v4"
)

// Get godoc
// @Summary      Get a specific kind box for a benefactor
// @Description  This endpoint retrieves a specific kind box associated with an authenticated benefactor.
// @Tags         KindBox
// @Accept       json
// @Produce      json
// @Param 		 id path int true "Kind box ID"
// @Success      200  {object}  param.KindBoxGetResponse
// @Failure      400  {string}  "Bad request"
// @Security 	 AuthBearerBenefactor
// @Router       /benefactor/kindboxes/{id} [get].
func (h Handler) Get(c echo.Context) error {
	var req param.KindBoxGetRequest
	if bErr := c.Bind(&req); bErr != nil {
		return echo.NewHTTPError(http.StatusBadRequest)
	}
	if fieldErrors, err := h.benefactorKindBoxVld.ValidateGetRequest(req); err != nil {
		msg, code := httpmsg.Error(err)

		return c.JSON(code, echo.Map{
			"message": msg,
			"errors":  fieldErrors,
		})
	}
	resp, sErr := h.benefactorKindBoxSvc.Get(c.Request().Context(), req)
	if sErr != nil {
		msg, code := httpmsg.Error(sErr)

		return echo.NewHTTPError(code, msg)
	}

	return c.JSON(http.StatusOK, resp)
}
