package agentkindboxvalidator

import (
	"errors"

	param "git.gocasts.ir/ebhomengo/niki/param/agent/kind_box"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	validation "github.com/go-ozzo/ozzo-validation/v4"
)

func (v Validator) ValidateReturn(req param.ReturnKindBoxRequest) (map[string]string, error) {
	const op = "agentkindboxvalidator.ValidateReturn"

	// TODO: add regex for serial number
	if err := validation.ValidateStruct(&req,
		validation.Field(&req.KindBoxID, validation.Required, validation.By(v.doesKindBoxExistForAgent(req.AgentID))),
		validation.Field(&req.SerialNumber, validation.Required),
	); err != nil {
		fieldErrors := make(map[string]string)

		var errV validation.Errors
		if errors.As(err, &errV) {
			for key, value := range errV {
				if value != nil {
					fieldErrors[key] = value.Error()
				}
			}
		}

		return fieldErrors, richerror.New(op).
			WithMessage(errmsg.ErrorMsgInvalidInput).
			WithKind(richerror.KindInvalid).
			WithMeta(map[string]interface{}{"req": req}).
			WithErr(err)
	}

	return map[string]string{}, nil
}
